(*

Make the figure comparing the exact numerical expectation of consumption growth
to what would be expected from a second- and a fourth-order Taylor expansion

*)

<<Graphics`Legend`

MakeEtetaList[xt_,LifePosn_] := (* Makes a list of the expected values of eta, eta^2, etc.  *)
Block[{}, 
  ctVal = ctSplice[xt,LifePosn];
  stVal = xt-ctVal;
  Sum[
      eta = 
      (G[[LifePosn+1]] PermShockVals[[j]])*
      ct[
        (RFunc[stVal]/(G[[LifePosn+1]]PermShockVals[[j]])) stVal+TranShockVals[[i]]
        ,LifePosn+1]/ctVal-1;
    {eta,eta^2,eta^3,eta^4}
    *TranShockProbs[[i]] PermShockProbs[[j]]
    ,{i,Length[TranShockVals]}
    ,{j,Length[PermShockVals]}
  ]
]; (* End MakeEtetaList *)

EtetaByOrder[xt_,LifePosn_,OrdersToReturn_] := (* Makes a list of expected consumption growth using up to 4th order Taylor expn *)
Block[{},
  EtetaList = MakeEtetaList[xt,LifePosn];
  cGrowthByOrder = {
   EtetaList[[1]]
  ,(1/rho)(1-1/(R beta))+((rho+1)/2)EtetaList[[2]]
  ,(1/rho)(1-1/(R beta))+((rho+1)/2)EtetaList[[2]]-((rho+1)(rho+2)/6)EtetaList[[3]]
  ,(1/rho)(1-1/(R beta))+((rho+1)/2)EtetaList[[2]]-((rho+1)(rho+2)/6)EtetaList[[3]]
   +((rho+1)(rho+2)(rho+3)/24)EtetaList[[4]]
  };
  Return[cGrowthByOrder[[OrdersToReturn]]]
];


cGrowthExactVsTaylorExpn = 
  Plot[
    Evaluate[EtetaByOrder[xt,LifePosn,{1,2,4}]]
    ,{xt,1.126,1.855}
, PlotStyle -> {Dashing[{}], Dashing[{.01}], Dashing[{.03}]}
, AxesOrigin -> {1.10, 0}
, PlotRange -> {{1.10, 1.85}, {0, .12}}
, PlotLegend -> {"True", "Second Order", "Fourth Order"}
, LegendSize -> {1.05, .3}
, LegendPosition -> {-.15, .05}
, Ticks -> (TickFunction[#1, #2, TextFunction -> TrimText]&)
, AxesLabel->{"x",""}
, ImageSize->HalfPageSize
, TextStyle->{FontFamily->"Times"}
]


If[SaveFigs == True,
  Export[SubDirectoryString<>"Figures"<>SubDirectoryString<>"cGrowthExactVsTaylorExpn.eps", cGrowthExactVsTaylorExpn, "EPS"];
];
