(*
Program to generate all the Mathematica results and figures in
"Death to the Log-Linearized Consumption Euler Equation"
*)

SeedRandom[1];                (* Initialize the random number generator so results will be reproducible *)

<<setup_workspace.m;          (* Set up a variety of background and housekeeping functions *)
<<make_data_file_function.m;  (* Create a function that can be used to save output results *)
<<solve_and_save_function.m;  (* Create function that solves a version of the model and saves the results *)
<<simulation_functions.m;     (* Define the functions that perform simulations *)

<<params_baseline.m;          (* Read in baseline parameter values *)
<<setup_grids.m;              (* Set up the discretization of the state space *)
<<multiperiod_vars.m;         (* Define the variables used in solving the problem *)
<<multiperiod_functions.m;    (* Define the functions used in solving the problem *)
<<setup_shocks.m;             (* Create the discrete approximations to the stochastic shock processes *)
SplicePoint = .50;            (* xt Point below which approximate c func is replaced with exact *)

ProgramDirectory = Directory[];

outputdir := SetDirectory[ProgramDirectory<>SubDirectoryString<>"output"];
programdir:= SetDirectory[ProgramDirectory];

SaveFigs = True;

NumToSimulate  = 1000; (* Num of consumers to sim for each parameter configuration *)
LastPeriod       =20;  (* Num of periods to continue the simulations *)
FirstPeriodToKeep=11;  (* Discard the first 10 periods during which dist approaches ergodic *)


<<params_baseline.m;

SolveIt["base"];
SolveAndSimAndSave["base"];
<<RealEuler.m;

<<params_baseline.m;
r = .0;
R = 1+r;
SolveAndSimAndSave["r00"];

<<params_baseline.m;
r = .04;
R = 1+r;
SolveAndSimAndSave["r04"];

<<params_baseline.m;
delta = .00;
beta = 1/(1+delta);
SolveAndSimAndSave["del00"];

<<params_baseline.m;
delta = .08;
beta = 1/(1+delta);
SolveAndSimAndSave["del08"];

<<params_baseline.m;
g = .02;
G = Table[1+g,{i,LifeLength+1}];
SolveAndSimAndSave["g02"];

<<params_baseline.m;
g = .06;
G = Table[1+g,{i,LifeLength+1}];
SolveAndSimAndSave["g06"];

<<params_baseline.m;
gamma = -.0001;
rho = 1-gamma;<<multiperiod_vars.m;<<multiperiod_functions.m;
SolveAndSimAndSave["rho1"];

<<params_baseline.m;
gamma = -4;
rho = 1-gamma;<<multiperiod_vars.m;<<multiperiod_functions.m;
SolveAndSimAndSave["rho5"];

<<params_baseline.m;
sigmaPerm = .05; PermShockDraws = {};<<setup_shocks.m;
SolveAndSimAndSave["ep05"];

<<params_baseline.m;
sigmaPerm = .15; PermShockDraws = {};<<setup_shocks.m;
SolveAndSimAndSave["ep15"];

<<params_baseline.m;
sigmaTran = .05; PermShockDraws = {}; TranShockDrawsZero = {};<<setup_shocks.m;
SolveAndSimAndSave["et05"];

<<params_baseline.m;
sigmaTran = .15; TranShockDrawsZero = {};<<setup_shocks.m;
SolveAndSimAndSave["et15"];

<<estimate_GMM.m;

<<cGrowthExactVsTaylorExpn.m; (* Make figure showing relationship between the true and approximated consumption growth functions *)


(* Now construct figure for CDF.  Baseline parameter values solution already in memory from estimate_GMM.m *)

NumToSimulate     = 20000;    (* Number of people to simulate *)
LastPeriod        = 20;       (* Simulate through 20 periods *)
FirstPeriodToKeep = 1;        (* Choose a value > 1 to save memory space *)

InitialSavings = 0.000001;
Simulate[NumToSimulate,LastPeriod,InitialSavings];
<<ActualVsErgodicCDF.m;       (* Make the figure comparing the ergodic dist to what you get after 10 periods *)

