
(* 
Set up data and then estimate the true (nonlinear) Euler equation using GMM
*)

<<setup_workspace.m;
<<make_data_file_function.m;
<<solve_and_save_function.m;
<<simulation_functions.m;

<<params_baseline.m;
<<setup_grids.m;
<<multiperiod_vars.m;
<<multiperiod_functions.m;
<<setup_shocks.m;
SplicePoint = .50;    (* xt Point at which approximate c func is replaced with exact *)

ProgramDirectory = Directory[];

outputdir := SetDirectory[ProgramDirectory<>SubDirectoryString<>"output"];
programdir:= SetDirectory[ProgramDirectory];

SaveFigs = True;

NumToSimulate     = 1000;
LastPeriod        = 20;
FirstPeriodToKeep = 1;

<<params_baseline.m;
SeedRandom[2];
SolveIt["base"];

FindxTarget; 
Simulate[NumToSimulate,LastPeriod,xTarget-1];

StartSamplePeriod = 1;
EndSamplePeriod   = 20;

cRatioMasterList       = Flatten[Table[Exp[cGrowthList[[i]]],{i,StartSamplePeriod,EndSamplePeriod}]];
InterestRateMasterList = Flatten[Table[InterestRateList[[i]],{i,StartSamplePeriod,EndSamplePeriod}]];

ListOfcRatios = cRatioMasterList;
ListOfInterestRates = InterestRateMasterList;

<<estimate_GMM_functions.m;

GMMEstimate;

SeedRandom[3];
GMMEstimateList = {};
GMMEstimateBeta0p98List = {};
GMMEstimateBeta1O1p08List = {};
GMMEstimateNoise1List = {};
GMMEstimateNoise2List = {};

MonteCarloSize = 1000;

VerboseOutput=False;
Do[
Print["Monte Carlo Sim # ",i];
RawDataSize = Length[cRatioMasterList];
WhichOnesToTake = Take[RandomPermutation[RawDataSize],1000];
ListOfcRatios = cRatioMasterList[[WhichOnesToTake]];
ListOfInterestRates = InterestRateMasterList[[WhichOnesToTake]];

If[VerboseOutput == True,Print["Solving Correctly."]];
GMMEstimate;
GMMEstimateList = Append[GMMEstimateList,{rhohat,rhose}];

If[VerboseOutput == True,Print["Solving with \[Beta] = 0.98"]];
beta = 0.98;
GMMEstimate;
GMMEstimateBeta0p98List = Append[GMMEstimateBeta0p98List,{rhohat,rhose}];

If[VerboseOutput == True,Print["Solving with \[Beta] = 1/1.08"]];
beta = 1/1.08;
GMMEstimate;
GMMEstimateBeta1O1p08List = Append[GMMEstimateBeta1O1p08List,{rhohat,rhose}];

beta = 0.96;
If[VerboseOutput == True,Print["Solving with Signal/Noise Ratio = 1/2"]];
cNoiseDraws1 = Take[RandomPermutation[RawDataSize],1000];
ListOfcRatios = ListOfcRatios*cRatioMasterList[[cNoiseDraws1]]/Mean[cRatioMasterList[[cNoiseDraws1]]];

GMMEstimate;
GMMEstimateNoise1List = Append[GMMEstimateNoise1List,{rhohat,rhose}];

If[VerboseOutput == True,Print["Solving with Signal/Noise Ratio = 1/3"]];
cNoiseDraws2 = Take[RandomPermutation[RawDataSize],1000];
ListOfcRatios = ListOfcRatios*cRatioMasterList[[cNoiseDraws2]]/Mean[cRatioMasterList[[cNoiseDraws2]]];

GMMEstimate;
GMMEstimateNoise2List = Append[GMMEstimateNoise2List,{rhohat,rhose}];

,{i,1,MonteCarloSize}];

GMMResults = OpenWrite["output"<>SubDirectoryString<>"gmm.txt",FormatType->OutputForm];

WriteNextLineOfResults[GMMResults,GMMEstimateList];
WriteNextLineOfResults[GMMResults,GMMEstimateNoise1List];
WriteNextLineOfResults[GMMResults,GMMEstimateNoise2List];
WriteNextLineOfResults[GMMResults,GMMEstimateBeta0p98List];
WriteNextLineOfResults[GMMResults,GMMEstimateBeta1O1p08List];

Close[GMMResults];
