(*
All of these functions require ListOfInterestRates and ListOfcRatios to be resident in global memory
*)

MeanEuler[rhoest_]        := Mean[ 1-(1+ListOfInterestRates) beta (ListOfcRatios^-rhoest)];
MeanEulerSquared[rhoest_] := Mean[(1-(1+ListOfInterestRates) beta (ListOfcRatios^-rhoest))^2];
MeanEulerPrime[rhoest_]   := Mean[-(1+ListOfInterestRates) beta (ListOfcRatios^-rhoest) Log[ListOfcRatios]];

GMMEstimate := Block[{},

Check[   (* Check whether it fails with any of the errors listed below *)
    rhohat = (rhoest /. FindRoot[MeanEuler[rhoest] == 0,{rhoest,{rho+0.5,rho-0.5}}])
  ,Print["Failed FindRoot"];Interrupt[],FindRoot::frsec,ReplaceAll::reps,FindRoot::fddis,FindRoot::frmp,FindRoot::frns
]; (*End Check *)

rhovar = (MeanEulerSquared[rhohat]/Length[ListOfcRatios])/(MeanEulerPrime[rhohat]^2);
rhose  = rhovar^.5;

If[VerboseOutput == True,
  Print["Estimated rho: ",rhohat];
  Print["Estimated SE:  ",rhose];
];
];


WriteNextLineOfResults[FileToWriteTo_,GMMEstimateResults_] := Block[{},
{CoeffEstimateList,StdErrEstimateList} = Transpose[GMMEstimateResults];

MeanCoeffEstimate  = Mean[CoeffEstimateList];
MeanStdErrEstimate = Mean[StdErrEstimateList];

CoeffEstimateList  = Sort[CoeffEstimateList];
StdErrEstimateList = Sort[StdErrEstimateList];

Pct05CoeffEstimate = CoeffEstimateList[[Round[.05*MonteCarloSize]]];
Pct95CoeffEstimate = CoeffEstimateList[[Round[.95*MonteCarloSize]]];

Pct05StdErrEstimate = StdErrEstimateList[[Round[.05*MonteCarloSize]]];
Pct95StdErrEstimate = StdErrEstimateList[[Round[.95*MonteCarloSize]]];

Write[FileToWriteTo,
      PaddedForm[MeanCoeffEstimate,{3,2}]," > & > [",PaddedForm[Pct05CoeffEstimate,{3,2}],",",PaddedForm[Pct95CoeffEstimate,{3,2}],"]"];
Write[FileToWriteTo,
  "(",PaddedForm[MeanStdErrEstimate,{3,2}],") > & > (",PaddedForm[Pct05StdErrEstimate,{3,2}],",",PaddedForm[Pct95StdErrEstimate,{3,2}],")"];
(*
Print["Results:"];
Print["{MeanCoeff ,0.05,0.95} = ",{MeanCoeffEstimate,Pct05CoeffEstimate,Pct95CoeffEstimate}];
Print["{MeanStdErr,0.05,0.95} = ",{MeanStdErrEstimate,Pct05StdErrEstimate,Pct95StdErrEstimate}];
*)

];
