(*

Defines the functions needed for solving the multiperiod optimization problem

*)

OmegatRaw[st_,LifePosn_] := 
  If[Chop[st] == 0 && ZeroProb>0, 
    (* then *) Return[-Infinity],
    (* else *) 
    If[st == sGrid[[1]] && LiqConstr == False, Return[-Infinity]];
    Return[
      Sum[
         TranShockProbs[[i]] PermShockProbs[[j]] *
         (G[[LifePosn+1]] PermShockVals[[j]])^(1-rho) *
         Vt[(RFunc[st]/(G[[LifePosn+1]] PermShockVals[[j]])) st+TranShockVals[[i]],LifePosn+1]
        ,{i,Length[TranShockVals]}
      ,{j,Length[PermShockVals]}
      ]
    ]
  ];

OmegaInvtRaw[st_,LifePosn_] := Block[{},
  If[LifePosn == LifeLength,Return[0]];
  OmegatRawVal = OmegatRaw[st,LifePosn];
  Return[ (OmegatToOmegaInvtMultiply*OmegatRawVal)^OmegatToOmegaInvtPower];
];

Omegat[st_,LifePosn_] := Block[{},
  If[LifePosn >= LifeLength,Return[0]];
  Return[(OmegaInvtInterpFunc[[LifePosn]][st]^(1/OmegatToOmegaInvtPower))
         /OmegatToOmegaInvtMultiply];
];


OmegaPrimetRaw[st_,LifePosn_] := 
  If[st < -PDVMinHtp1[[LifePosn]],
    (* then *) Return[Infinity],
    (* else *) Return[
      Sum[
        TranShockProbs[[i]] PermShockProbs[[j]]*
        (RFunc[st](G[[LifePosn+1]] PermShockVals[[j]])^(-rho))*
        VPrimet[
          (RFunc[st]/(G[[LifePosn+1]]PermShockVals[[j]])) st+TranShockVals[[i]]
          ,LifePosn+1]
      ,{i,Length[TranShockVals]}
      ,{j,Length[PermShockVals]}
      ]
    ]
  ];
  
OmegaPrimetRawList[st_,LifePosn_] :=  (* Useful for debugging purposes *)
  If[st < -PDVMinHtp1[[LifePosn]],
    (* then *) Return[Infinity],
    (* else *) Return[
      Table[{
        {TranShockVals[[i]],PermShockVals[[j]]},
        TranShockProbs[[i]] PermShockProbs[[j]],
        RFunc[st](G[[LifePosn+1]] PermShockVals[[j]])^(-rho),
        VPrimet[
          (RFunc[st]/(G[[LifePosn+1]]PermShockVals[[j]])) st+TranShockVals[[i]]
          ,LifePosn+1]
          }
      ,{i,Length[TranShockVals]}
      ,{j,Length[PermShockVals]}
      ]
    ]
  ];
  

OmegaInvPrimetRaw[st_,LifePosn_] := Block[{},
  If[LifePosn == LifeLength,Return[0]];
  If[Chop[st] == 0 && ZeroProb>0,Return[0]];
  If[st <= sGrid[[1]] && LiqConstr == False,Return[0]];
  OmegaPrimetRawVal = OmegaPrimetRaw[st,LifePosn];
  Return[(OmegaPrimetToOmegaInvPrimetMultiply OmegaPrimetRawVal)^OmegaPrimetToOmegaInvPrimetPower]
];

OmegaInvPrimet[st_,LifePosn_] := Block[{},
  If[LifePosn == LifeLength,Return[0]];
  If[Chop[st] == 0 && ZeroProb>0,Return[0]];
  If[st <= sGrid[[1]] && LiqConstr == False,Return[0]];
  Return[OmegaInvPrimetInterpFunc[[LifePosn]][st]];
];


OmegaPrimet[st_,LifePosn_] := Block[{},
  If[LifePosn >= LifeLength,Return[0]];
  If[st == 0 && ZeroProb>0,Return[-Infinity]];
  Return[(OmegaInvPrimetInterpFunc[[LifePosn]][st]^(1/OmegaPrimetToOmegaInvPrimetPower))
         /OmegaPrimetToOmegaInvPrimetMultiply]
];


ctRaw[xt_,LifePosn_] := Block[{},
  xtVal = xt;
  If[LifePosn >= LifeLength,Return[xt]];              (* Consume everything in last period of life *)
  If[xt <= -MaxDebt, Return[0]];                      (* If they are maximally indebted, they can't consume anything *)
  If[Chop[xt] == 0 && ZeroProb>0,Return[0]];          (* If zero resources, have no choice but to consume zero if ZeroProb>0 *)
  If[xt==xGrid[[1]] && LiqConstr==False, Return[0]];  (* If at max borrowing limit, can't consume *)
  If[LiqConstr == True,                               (* If borrowing constraint is binding *)
    (* then *)
      BindingcPoint = (beta^(1/-rho))OmegaInvPrimet[-MaxDebt,LifePosn];
      BindingxPoint = BindingcPoint-MaxDebt;
      If[xt <= BindingxPoint,Return[xt+MaxDebt]];
  ];
  If[Rborrow != R && Min[PDVMinHtp1[[LifePosn]],MaxDebt]>0,  (* If credit cards are available and borrowing is possible *)
    If[LowercBindingPoint <= xt <= UppercBindingPoint, Return[xt]]];
  ctp1Val = ct[xt,LifePosn+1];                     (* Figure out what you would do in this situation next period *)
  If[ctp1Val <= 0,ctp1Val = .5 (xt+Min[PDVMinHtp1[[LifePosn]],MaxDebt])];  
  ctSearchMax = Min[ctp1Val*1.05,xt+Min[PDVMinHtp1[[LifePosn]],MaxDebt]];
  ctSearchMin = Max[0,ctp1Val*.95];
  If[ZeroProb>0, ctSearchMax = Min[.999 xt,ctSearchMax]]; (* If ZeroProb > 0, don't allow them to consume more than current xt *)
  Check[   (* Check whether it fails with any of the errors listed below *)
  ctResults =
    FindRoot[
      ctOpt ==beta^(-1/rho) OmegaInvPrimet[xt-ctOpt,LifePosn]
    ,{ctOpt,{ctSearchMin,ctSearchMax}}
    ,AccuracyGoal->4];
    ,Print["Failed FindRoot"];Interrupt[],FindRoot::frsec,ReplaceAll::reps,FindRoot::fddis,FindRoot::frmp,FindRoot::frns];
  ctUnconstrained = (ctOpt /. ctResults);
  If[LiqConstr == False, Return[ctUnconstrained]];
  If[ctUnconstrained>xt+MaxDebt Rborrow,xt+MaxDebt Rborrow,ctUnconstrained]
];
  

ctRawRaw[xt_,LifePosn_] := Block[{}, (* Version that uses OmegaInvPrimetRaw rather than OmegaInvPrimet; useful for debugging *)
  xtVal = xt;
  If[LifePosn >= LifeLength,Return[xtVal]];              (* Consume everything in last period of life *)
  If[xtVal <= -MaxDebt, Return[0]]; 
  If[Chop[xtVal] == 0 && ZeroProb>0,Return[0]];          (* If zero resources, have no choice but to consume zero if ZeroProb>0 *)
  If[xtVal==xGrid[[1]] && LiqConstr==False, Return[0]];  (* If at max borrowing limit, can't consume *)
  If[LiqConstr == True,                               (* If borrowing constraint is binding *)
    (* then *)
      BindingcPoint = (beta^(1/-rho))OmegaInvPrimet[-MaxDebt,LifePosn];
      BindingxPoint = BindingcPoint-MaxDebt;
      If[xtVal <= BindingxPoint,Return[xtVal+MaxDebt]];
  ];
  If[Rborrow != R && Min[PDVMinHtp1[[LifePosn]],MaxDebt]>0,  (* If credit cards are available and borrowing is possible *)
    If[LowercBindingPoint <= xtVal <= UppercBindingPoint, Return[xtVal]]];
  ctp1Val = ct[xtVal,LifePosn+1];                     (* Figure out what you would do in this situation next period *)
  If[ctp1Val <= 0,ctp1Val = .5 (xtVal+Min[PDVMinHtp1[[LifePosn]],MaxDebt])];  
  ctSearchMax = Min[ctp1Val*1.05,xtVal+Min[PDVMinHtp1[[LifePosn]],MaxDebt]];
  ctSearchMin = Max[0,ctp1Val*.95];
(*   If[ZeroProb>0, ctSearchMax = Min[.9999 xtVal,ctSearchMax]]; (* If ZeroProb > 0, don't allow them to consume more than current xt *)*)
  Check[ 
  ctResults =
    FindRoot[
      ctOpt ==beta^(-1/rho) OmegaInvPrimetRaw[xtVal-ctOpt,LifePosn]
    ,{ctOpt,{ctSearchMin,ctSearchMax}}
    ,AccuracyGoal->6];
    ,Print["Failed FindRoot"];Interrupt[],FindRoot::frsec,ReplaceAll::reps,FindRoot::fddis];
  ctUnconstrained = (ctOpt /. ctResults);
  If[LiqConstr == False, Return[ctUnconstrained]];
  If[ctUnconstrained>xtVal+MaxDebt Rborrow,xtVal+MaxDebt Rborrow,ctUnconstrained]
];
    
ct[xt_,LifePosn_] := ctInterpFunc[[LifePosn]][xt];


VInvtRaw[xt_,LifePosn_] := Block[{},
  If[Chop[xt] == 0 && ZeroProb > 0, Return[0]];
  If[xt == xGrid[[1]] && LiqConstr == False,Return[0]];
  ctVt = ct[xt,LifePosn];
  VtVal = u[ctVt] + beta Omegat[xt-ctVt,LifePosn]; 
  Return[ (VtVal*VtToVInvtMultiply)^VtToVInvtPower];
];

Vt[xt_,LifePosn_] := Block[{},
  If[LifePosn > LifeLength,Return[0]];
  If[LifePosn == LifeLength,Return[u[xt]]];
  If[xt == 0 && ZeroProb>0,Return[-Infinity]];
  ctVt = ct[xt,LifePosn];
  Return[u[ctVt] + beta Omegat[xt-ctVt,LifePosn]];
];


VInvPrimetRaw[xt_,LifePosn_] := Block[{},
  ctVPrimet = ct[xt,LifePosn];
  Return[ctVPrimet]
];

VPrimet[xt_,LifePosn_] := Block[{},
  If[LifePosn >  LifeLength,Return[0]];
  If[LifePosn == LifeLength,Return[uprime[xt]]];
  If[Chop[xt] == 0 && ZeroProb>0,Return[Infinity]];
  Return[uprime[ct[xt,LifePosn]]]
];

EtLogCtp1OCt[xt_,LifePosn_] := Block[{},
  ctVal = ct[xt,LifePosn];
  stVal = xt-ctVal;
  Return[
  Sum[
    TranShockProbs[[i]] PermShockProbs[[j]]*
    Log[
      (G[[LifePosn+1]] PermShockVals[[j]])*
      ct[
        (RFunc[stVal]/(G[[LifePosn+1]]PermShockVals[[j]])) stVal+TranShockVals[[i]]
        ,LifePosn+1]
       ] (* End Log *)
    ,{i,Length[TranShockVals]}
    ,{j,Length[PermShockVals]}
  ] - Log[ctVal]
 ]; (* End Return *)
]; (* End EtLogCtp1OCt *)

EtLogCtp1OCtsq[xt_,LifePosn_] := Block[{},
  ctVal = ct[xt,LifePosn];
  stVal = xt-ctVal;
  Return[
  Sum[
    TranShockProbs[[i]] PermShockProbs[[j]]*
    (Log[
      (G[[LifePosn+1]] PermShockVals[[j]])*
      ct[
        (RFunc[stVal]/(G[[LifePosn+1]]PermShockVals[[j]])) stVal+TranShockVals[[i]]
        ,LifePosn+1]
       ] - Log[ctVal])^2 (* End Log *)
    ,{i,Length[TranShockVals]}
    ,{j,Length[PermShockVals]}
  ] 
 ]; (* End Return *)
]; (* End EtLogCtp1OCt *)


EtCtp1OCt[xt_,LifePosn_] := Block[{},
  ctVal = ct[xt,LifePosn];
  stVal = xt-ctVal;
  Return[
  Sum[
    TranShockProbs[[i]] PermShockProbs[[j]]*
      (G[[LifePosn+1]] PermShockVals[[j]])*
      ct[
        (RFunc[stVal]/(G[[LifePosn+1]]PermShockVals[[j]])) stVal+TranShockVals[[i]]
        ,LifePosn+1]
    ,{i,Length[TranShockVals]}
    ,{j,Length[PermShockVals]}
  ]/ctVal
 ]; (* End Return *)
]; (* End EtLogCtp1OCt *)


Euler[xt_,LifePosn_] := Block[{},  (* Calculate the expected value of the Euler equation *)
  ctVal = ctSplice[xt,LifePosn];
  stVal = xt-ctVal;
  Return[(R beta)*
  Sum[
    TranShockProbs[[i]] PermShockProbs[[j]]*
      (
      (G[[LifePosn+1]] PermShockVals[[j]])*
      ct[
        (RFunc[stVal]/(G[[LifePosn+1]]PermShockVals[[j]])) stVal+TranShockVals[[i]]
        ,LifePosn+1]/ctVal
     )^-rho
    ,{i,Length[TranShockVals]}
    ,{j,Length[PermShockVals]}
  ]
 ]; (* End Return *)
]; (* End Euler *)

OmegaPrimetRawRaw[st_,LifePosn_] := Block[{},
  Return[
  Sum[
  (*
    {TranShockVals[[i]],PermShockVals[[j]]},,*)
    TranShockProbs[[i]] PermShockProbs[[j]]*
      RFunc[st] ((G[[LifePosn+1]] PermShockVals[[j]])^(-rho))*
      (
      ct[
        (RFunc[st]/(G[[LifePosn+1]]PermShockVals[[j]])) st+TranShockVals[[i]]
        ,LifePosn+1]
     )^-rho (* } *)
    ,{i,Length[TranShockVals]}
    ,{j,Length[PermShockVals]}
  ]
 ]; (* End Return *)
]; (* End Euler *)


Etxtp1[xt_,LifePosn_] := Block[{},
  ctEtxtp1 = ct[xt,LifePosn];
  stEtxtp1 = xt-ctEtxtp1;
  Return[
  Sum[
    TranShockProbs[[i]] PermShockProbs[[j]]*
        ((RFunc[stEtxtp1]/(G[[LifePosn+1]]PermShockVals[[j]])) stEtxtp1+TranShockVals[[i]])
    ,{i,Length[TranShockVals]}
    ,{j,Length[PermShockVals]}
  ]
 ];
];

FindxTarget := Block[{},
  Return[xt /. FindRoot[Etxtp1[xt, LifePosn] == xt, {xt, {1, 2}}]
  ]
];

MPC[xt_,LifePosn_] := (ct[xt+.0001,LifePosn]-ct[xt,LifePosn])/0.0001;

ctSplice[xt_,LifePosn_] := If[xt>SplicePoint,ct[xt,LifePosn],ctRawRaw[xt,LifePosn]];
