(*

Defines a function which solves the optimization problem under a set of 
parameter values assumed resident in memory, and saves the results 
in the "output" directory.

Global variables that need to be defined when this function is called:

ZeroProb    - probability of a zero-income event
sigmaTran   - std deviation of the lognormally distributed transitory shock
sigmaPerm   - std deviation of the lognormally distributed transitory shock

FreshSolve  - whether to try loading the solved rules from disk, or solve afresh
*)

SolveAndSim[NameOfExperiment_] := Block[{},

Print["Solving "<>NameOfExperiment];

SolveIt[NameOfExperiment];

Print["Simulating "<>NameOfExperiment];

Simulate[NumToSimulate,LastPeriod,xTarget-1];

];

SolveAndSimAndSave[NameOfExperiment_] := Block[{},

SolveAndSim[NameOfExperiment];

DummyVarName = "dum"<>NameOfExperiment;
Print["Saving "<>NameOfExperiment];
outputdir;
MakeDataFile[NameOfExperiment<>".raw"];
programdir;
];


SolveIt[NameOfExperiment_] := Block[{},

{TranShockVals,TranShockProbs} = DiscreteApproxToMeanOneLogNormal[sigmaTran,NumOfTranShockPoints];
{PermShockVals,PermShockProbs} = DiscreteApproxToMeanOneLogNormal[sigmaPerm,NumOfPermShockPoints];

If[ZeroProb>0,
  (* then modify the shocks and probabilities to incorporate the zero risk *)
  TranShockVals = TranShockVals/(1-ZeroProb);
  TranShockVals = Prepend[TranShockVals,0];
  TranShockProbs = TranShockProbs*(1-ZeroProb);
  TranShockProbs = Prepend[TranShockProbs,ZeroProb];
];

UnempProb = 0;      (* Unemployment with benefits does not exist in this exercise *)
LiqConstr = False;  (* No liquidity constraints *)
MaxDebt = 0;        (* No borrowing *)

<<solve_multiperiod.m; 

xTarget = FindxTarget;

];
