(*

Assuming preferences and necessary functions are resident in memory, solve backwards
from last period of life to LifePosn=1 

*)

LifePosn = LifeLength;  (* Set LifePosn indicator value to the last period of life *)

<<impatience.m;

While[LifePosn>0,

Print[LifePosn];

If[LiqConstr == False && MaxDebt > 0,sGrid = sGridBaseline - Min[PDVMinHtp1[[LifePosn]],MaxDebt]];  (* Allow repayable borrowing *)

MakeArgArrays[sFunctions];      (* Make lists of arguments for the functions that take end-of-period savings as prime argument *)

(* First make a linear interpolation *)
OmegaInvPrimetArgumentList           = AddOutcome[OmegaInvPrimetArgArray,Table[LifePosn,{Length[OmegaInvPrimetArgArray]}]];
OmegaInvPrimetRawResults             = Map[Apply[OmegaInvPrimetRaw,#] &,OmegaInvPrimetArgumentList];(*Calculate for each argument*)
OmegaInvPrimetInterpData[[LifePosn]] = AddOutcome[OmegaInvPrimetArgArray,OmegaInvPrimetRawResults]; (*Construct {Argument,Result}*)
OmegaInvPrimetInterpFunc[[LifePosn]] = Interpolation[OmegaInvPrimetInterpData[[LifePosn]],InterpolationOrder->1];

(* Now add three points beyond the end of the linear interpolation, using that linear interpolation itself *)
OmegaInvPrimetInterpData[[LifePosn]] = 
    Append[
      OmegaInvPrimetInterpData[[LifePosn]]
      ,Flatten[{OmegaInvPrimetArgArray[[-1]]+1,Apply[OmegaInvPrimet,OmegaInvPrimetArgumentList[[-1]]+{1,0}]}]];
OmegaInvPrimetInterpData[[LifePosn]] = 
    Append[OmegaInvPrimetInterpData[[LifePosn]]
      ,Flatten[{OmegaInvPrimetArgArray[[-1]]+10,Apply[OmegaInvPrimet,OmegaInvPrimetArgumentList[[-1]]+{10,0}]}]];
OmegaInvPrimetInterpData[[LifePosn]] = Append[OmegaInvPrimetInterpData[[LifePosn]]
      ,Flatten[{OmegaInvPrimetArgArray[[-1]]+100,Apply[OmegaInvPrimet,OmegaInvPrimetArgumentList[[-1]]+{100,0}]}]];

(* 
Now do a third-order interpolation through the new datset.  The added points on the end make the 3rd order linear beyond
the original endpoint, but allow nonlinearity below the end point 
*)
OmegaInvPrimetInterpFunc[[LifePosn]] = Interpolation[OmegaInvPrimetInterpData[[LifePosn]],InterpolationOrder->3];

xGrid = xGridBaseline;
If[LiqConstr==False || MaxDebt>0,xGrid=xGridBaseline-Min[PDVMinHtp1[[LifePosn]],MaxDebt]];  (* Start grid at lowest possible x *)

(* Now find the point(s) at which the liquidity constraint(s) (if any) begin(s) to bind *)
If[LiqConstr == True, (* Liquidity constraints only bind if ZeroProb = 0 *)
  If[LifePosn < LifeLength,
    If[MaxDebt < PDVMinHtp1[[LifePosn]],
      (* then the max they are allowed to borrow is < max they could pay back, implying a bindingpoint *)
      BindingcPoint = (beta^(1/-rho))OmegaInvPrimet[-MaxDebt,LifePosn];
      BindingxPoint = BindingcPoint-MaxDebt;
      xGrid = Union[{BindingxPoint},xGrid];  (* Add the point to the gridpoints *)
      ]; (* End If *)      
    If[Rborrow != R && Min[PDVMinHtp1[[LifePosn]],MaxDebt]>0,  (* If credit cards are available and borrowing is possible *)
      (* then the consumption function will lie atop the 45 degree line over some range *)
      UppercBindingPoint = (beta^(1/-rho))OmegaInvPrimetInterpFunc[[LifePosn]][0];
      LowercBindingPoint = (beta^(1/-rho))OmegaInvPrimetInterpFunc[[LifePosn]][-epsilon];
      xGrid = Union[{UppercBindingPoint,LowercBindingPoint},xGrid]];
  ];
];

MakeArgArrays[xFunctions];                    (* Reconstruct the ArgArrays using redefined xGrid *)

ctArgumentList           = AddOutcome[ctArgArray,Table[LifePosn,{Length[ctArgArray]}]];
ctResults                = Map[Apply[ctRaw,#] &,ctArgumentList];
ctInterpData[[LifePosn]] = AddOutcome[ctArgArray,ctResults];
ctInterpFunc[[LifePosn]] = Interpolation[ctInterpData[[LifePosn]],InterpolationOrder->1];

(* Same trick as above to allow third order interpolation below max point and linear above it *)
ctInterpData[[LifePosn]] = Append[ctInterpData[[LifePosn]],Flatten[{ctArgArray[[-1]]+1,Apply[ct,ctArgumentList[[-1]]+{1,0}]}]];
ctInterpData[[LifePosn]] = Append[ctInterpData[[LifePosn]],Flatten[{ctArgArray[[-1]]+10,Apply[ct,ctArgumentList[[-1]]+{10,0}]}]];
ctInterpData[[LifePosn]] = Append[ctInterpData[[LifePosn]],Flatten[{ctArgArray[[-1]]+100,Apply[ct,ctArgumentList[[-1]]+{100,0}]}]];
ctInterpFunc[[LifePosn]] = Interpolation[ctInterpData[[LifePosn]],InterpolationOrder->3];

LifePosn--;
If[LifePosn>0 && MaxDebt > 0,PDVMinHtp1[[LifePosn]] = (G[[LifePosn+1]] PermShockVals[[1]]/Rborrow)( PDVMinHtp1[[LifePosn+1]]+ TranShockVals[[1]])];

] (* End While *)

LifePosn++;
